/**
 * Created by lcg on 16/9/4.
 */

import { FileType } from './lib/fileType';
import FetchManager from './lib/fetchManager';
import SaveManager from './lib/saveManager';
import RemoveManager from './lib/removeManager';
import ClearManager from './lib/clearManager';
import CacheManager from './lib/cacheManager';
import utils from './lib/utils';
import uuid from 'uuid';

class FileCacheManager {
    _fetchManager = new FetchManager();
    _saveManager = new SaveManager();
    _removeManager = new RemoveManager();
    _clearManager = new ClearManager();


    /**
     * 获取本地缓存
     * @param uri
     * @returns {Promise} 为空时,代表缓存不存在.
     */
    exist(uri){
        uri = utils.encode(uri);
        return CacheManager.getPath(uri);
    }

    /**
     * 获取缓存路径,如果uri缓存不存在,则下载并返回本地路径.
     * @param {string} uri 远程地址
     * @param {object} option 参数
     *          @property {FileType} type 可选,默认为other, 目前定义在FileType中,理论上,任何字符串都可以,或根据不同type创建不同的文件夹.
     *          @property {string} method 可选,默认为GET, 可选值GET,POST
     *          @property {string} ext 可选,默认为空,缓存到本地的文件追加的扩展名.
     *          @property {function} onProgress 可选,下载进度.
     *          @property {function} onUploadProgress 可选,上传进度.
     *          @property {object} headers 可选,请求头
     *          @property {object} data 可选,post时携带的数据,headers及data参见react-native-fetch-blob文档
     * @returns {Promise} 返回值可调用cancel方法取消任务.
     */
    fetch(uri, option) {
        uri = utils.encode(uri);
        let taskId = this._taskId();
        let self = this;

        option = option || {};
        option.type = option.type || FileType.Other;

        let promise = new Promise((resolve, reject) => {
            return self._fetchManager.fetch(taskId, uri, option, resolve, reject);
        });

        promise.cancel = () => {
            this._fetchManager.cancel(uri, taskId);
        };

        return promise;
    }

    /**
     * 保存本地文件到缓存.
     * @param {string} uri 本地文件上传到服务器后所指向的url
     * @param {string} source 本地路径
     * @param {object} option 参数
     *          @property {FileType} type 可选,默认为other, 目前定义在FileType中,理论上,任何字符串都可以,或根据不同type创建不同的文件夹.
     *          @property {string} ext 可选,默认为空,缓存到本地的文件追加的扩展名.
     * @returns {Promise} 返回值可调用cancel方法取消任务.
     */
    save(uri, source, option) {
        uri = utils.encode(uri);
        let taskId = this._taskId();
        let self = this;

        let promise = new Promise((resolve, reject) => {
            return self._saveManager.save(taskId, uri, source, option, resolve, reject);
        });

        promise.cancel = () => {
            self._saveManager.cancel(uri, taskId);
        };

        return promise;
    }

    /**
     * 获取缓存列表
     * @param type  缓存类型
     */
    list(type) {
        return CacheManager.getList(type);
    }

    /**
     * 删除uri所指向的缓存文件
     * @param uri
     * @returns {Promise}
     */
    remove(uri) {
        uri = utils.encode(uri);
        let taskId = this._taskId();
        let self = this;

        let promise = new Promise((resolve, reject) => {
            return self._removeManager.remove(taskId, uri, resolve, reject);
        });

        promise.cancel = () => {
            self._removeManager.cancel(uri, taskId);
        };

        return promise;
    }

    /**
     * 清空缓存
     * @param type 缓存类型
     * @returns {Promise}
     */
    clear(type) {
        let taskId = this._taskId();
        let self = this;

        let promise = new Promise((resolve, reject) => {
            return self._clearManager.clear(taskId, type, resolve, reject);
        });

        promise.cancel = () => {
            self._clearManager.cancel(type, taskId);
        };

        return promise;
    }

    _taskId() {
        // let buffer = new Array(32);
        // uuid.v4(null, buffer, 0);
        // return uuid.unparse(buffer)
        return uuid();
    }
}

let FileCache = new FileCacheManager();

export { FileCache, FileType };